--------------------------------------------------------------------------------
-- Company: 
-- Engineer:
--
-- Create Date:    14:42:35 06/16/05
-- Design Name:    
-- Module Name:    top - Behavioral
-- Project Name:   
-- Target Device:  
-- Tool versions:  
-- Description:
--
-- Dependencies:
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top is
    Port ( clk : in std_logic;
           ireset : in std_logic;
           lcd_db : out std_logic_vector(7 downto 0);
           lcd_rs : out std_logic;
           lcd_rw : out std_logic;
			  resetout : out std_logic;
           lcd_e : out std_logic);
end top;

architecture Behavioral of top is

component lcd is
    Port ( clk 		: in std_logic;
           reset 		: in std_logic;
			  DB			: in std_logic_vector(7 downto 0);
			  W			: in std_logic;
			  Ready	 	: inout std_logic;
           lcd_rs 	: out std_logic;
           lcd_rw 	: out std_logic;
           lcd_e 		: out std_logic;
           lcd_db 	: out std_logic_vector(7 downto 0));
end component;

signal lcd_w	: std_logic;
signal reset	: std_logic;
signal lcd_ready	: std_logic;
signal CharBus 	: std_logic_vector(7 downto 0);
signal count 	: std_logic_vector(3 downto 0);
--**************************************************************************
-- STATE MACHINE SIGNAL DECLARATION:
type StateType is (
							Standby,
							WriteCommand,
							WriteData,
							stop   
							);

signal CurrentState, NextState : StateType;
--**************************************************************************

begin

resetout <= ireset;
reset <= not ireset;

lcd_module: lcd
    Port map(  clk 		=> clk,
           		reset 	=> reset,
			  		DB			=> CharBus,
			  		W			=> lcd_w,
			  		Ready	 	=> lcd_ready,
           		lcd_rs 	=> lcd_rs,
           		lcd_rw 	=> lcd_rw,
           		lcd_e 	=> lcd_e,
           		lcd_db 	=> lcd_db);

process(lcd_ready, reset)
begin
	if(reset = '1') then
		count <= (others=> '0');
	elsif(lcd_ready'event and lcd_ready = '1') then
		count <= count + 1;
	end if;
end process;

COMB: process(CurrentState, lcd_ready, count)
begin
	case CurrentState is

		when Standby =>
			if(lcd_ready = '1') then		  
				NextState <= WriteCommand;
			else NextState <= Standby;
			end if;

		when WriteCommand =>
			NextState <= WriteData;

		when WriteData =>
			if(count = 12) then		  
				NextState <= stop;
			else NextState <= standby;
			end if;

		when stop =>
			NextState <= stop;

	end case;
end process COMB;

SEQ: process(reset, clk)
begin
	if(reset = '1') then
		CurrentState <= standby;
	elsif (clk'event and clk = '1') then
		CurrentState <= NextState;
	end if;
end process SEQ;

with CurrentState select
	lcd_w <= '1' when WriteCommand,
			   '1' when WriteData,
			   '0' when Others;

	CharBus <= 
				x"4D" when count = 1 else
				x"41" when count = 2 else
				x"52" when count = 3 else
				x"4B" when count = 4 else
				x"3D" when count = 5 else
				x"41" when count = 6 else
				x"53" when count = 7 else
				x"53" when count = 8 else
				x"48" when count = 9 else
				x"4F" when count = 10 else
				x"4C" when count = 11 else
			   x"45" ;

end Behavioral;
